@@ -1,4 +1,19 @@
 from lxml.objectify import fromstring
+import binascii
+import zlib
+import six
+
+
+def is_gziped(content):
+    signature = content[:2]
+    if isinstance(signature, six.binary_type):
+        res = binascii.hexlify(signature) == b'1f8b'
+    elif isinstance(signature, six.text_type):
+        res = binascii.hexlify(signature.encode('utf-8')) == b'1f8b'
+    else:
+        raise ValueError('type {} not supported'.format(type(content)))
+
+    return res
 
 
 class BaseMessage(object):
@@ -33,6 +48,8 @@ class BaseMessage(object):
 
         if hasattr(value, 'read'):
             value = value.read()
+        if is_gziped(value):
+            value = zlib.decompress(value, zlib.MAX_WBITS | 32)
         self._xml = value
         self._objectified = fromstring(self._xml)
 
